#!/usr/bin/perl -w

use strict;

my %ins;
sub reg {
    my ($name) = @_;
    if ($name =~ /^in(\d+)$/) {
	($1 >= 0 && $1 <= 5) or die "cannot handle $name";
	return 3+$1; # map i0-i5 to $3-$8
    } elsif ($name =~ /^out(\d+)$/) {
	($1 >= 0 && $1 <= 3) or die "cannot handle $name";
	return 9+$1; # map out0-out3 to $9-$12
    } elsif ($name =~ /^\*out(\d+)$/) {
	($1 >= 0 && $1 <= 3) or die "cannot handle $name";
	return 13+$1; # map *out0-*out3 to $13-$16
    } elsif ($name =~ /^tmp(\d+)$/) {
	($1 >= 6 && $1 <= 68) or die "cannot handle $name";
	return 11+$1; # map tmp6-tmp68 to $17-$79
    }
    die "no register for $name";
}

scalar(@ARGV) == 1 or die "Usage: $0 <sN.verbose>\n";
my $fname = $ARGV[0];
$fname =~ /^s(\d+)(\..*)/ or die "cannot extract S-box number from $fname\n";
my $sbox = $1;
my $fname2 = "s$sbox.s";
my ($fh, $fh2);
my %tmpX2out; # maps the tmpX variables to output bits
my %tmpX2in; # maps the tmpX variables to input bits
open($fh, "<", $fname) or die "open: $fname: $!\n";
open($fh2, ">", $fname2) or die "open: $fname2: $!\n";
print { $fh2 } ".text\n.balign 8\n\n";
while (<>) {
    if (m{^Achieved (\d+) gates.*}) {
	print { $fh2 } "# gate count: $1\n".
	".global s$sbox\n.type s$sbox, \@function\ns$sbox:\n";
	printf { $fh2 } ("\tlqd \$%d,0(\$%d)\n", reg("*out$_"), reg("out$_"))
	foreach (0..3);
	printf { $fh2 } "\thbr .L$sbox,\$lr\n";
    } elsif (/^circuit.*outs:(\d+),(\d+),(\d+),(\d+)/) {
	%tmpX2out = ($1 => 0, $2 => 1, $3 => 2, $4 => 3);
    } elsif (/^ +([0-5]): 0x.{16} \(i([0-5])\)$/) {
	$tmpX2in{$1} = $2;
    } elsif (/^ +(\d+): 0x.{16} (\w+) ([\d,]+)$/) {
	my $tmpX = $1;
	my $ins = $2;
	my @args = split /,/, $3;
	$ins = 'xori' if $ins eq 'not'; # handles not
	printf { $fh2 } "\t%s \$%d,", $ins, reg("tmp$tmpX");
	# output the registers (can be input bits...)
	my @regs;
	foreach my $arg (@args) {
	    if (exists($tmpX2in{$arg})) {
		push @regs, '$'.reg("in$tmpX2in{$arg}");
	    } else {
		push @regs, '$'.reg("tmp$arg");
	    }
	}
	push @regs, -1 if $ins eq 'xori'; # handles not
	printf { $fh2 } "%s\n", join(",", @regs);
	# if output bits...
	if (exists($tmpX2out{$tmpX})) {
	    printf { $fh2 } "\txor \$%d,\$%d,\$%d\n",
	    reg("*out$tmpX2out{$tmpX}"), reg("*out$tmpX2out{$tmpX}"),
	    reg("tmp$tmpX");
	    printf { $fh2 } "\tstqd \$%d,0(\$%d)\n",
	    reg("*out$tmpX2out{$tmpX}"), reg("out$tmpX2out{$tmpX}");
	}
    } else {
	die "line $.: cannot parse: $_";
    }
}
print { $fh2 } ".L$sbox:\tbi \$lr\n";
close($fh) or die "close: $fname: $!\n";
close($fh2) or die "close: $fname2: $!\n";
